<?php

namespace Drupal\cmlapi\Service;

use Symfony\Component\Yaml\Yaml;

/**
 * Class ParserBase.
 */
class ParserBase {

  /**
   * Constructs a new ParserBase object.
   *
   * @param \Drupal\cmlapi\Service\CmlServiceInterface $cml
   *   The alias cleaner.
   * @param \Drupal\cmlapi\Service\XmlParserInterface $xml_parser
   *   The alias storage helper.
   */
  public function __construct(CmlServiceInterface $cml, XmlParserInterface $xml_parser) {
    $this->cmlService = $cml;
    $this->xmlParserService = $xml_parser;
  }

  /**
   * Get Rows (Chunked cache).
   */
  public function getRows($filepath, $skip_cache = FALSE) {
    $rows = &drupal_static("TovarParcer::getRows():$filepath");
    $size = 300;
    if (!isset($rows)) {
      $cache_key = 'TovarParcer:' . $filepath;
      if ($skip_cache) {
        $cache_key .= rand();
      }
      if ($cache = \Drupal::cache()->get($cache_key)) {
        $rows = [
          'info' => \Drupal::cache()->get("$cache_key::info")->data,
          'data' => [],
        ];
        if (is_numeric($cache->data)) {
          $chunks = intdiv($cache->data, $size);
          for ($i = 0; $i <= $chunks; $i++) {
            $chunk = \Drupal::cache()->get("$cache_key::data::$i")->data;
            $rows['data'] = array_merge($rows['data'], $chunk);
          }
        }
      }
      else {
        if ($filepath) {
          $this->xmlParserService->parseXmlFile($filepath);
          $data = self::parce($this->xmlParserService->xmlString);
          if (!empty($data)) {
            $rows = $data;
          }
        }
        \Drupal::cache()->set("$cache_key::info", $rows['info']);
        if (isset($rows['data'])) {
          $count = count($rows['data']);
          \Drupal::cache()->set($cache_key, $count);
          $chunks = array_chunk($rows['data'], $size, TRUE);
          foreach ($chunks as $i => $chunk) {
            \Drupal::cache()->set("$cache_key::data::$i", $chunk);
          }
        }
      }
    }
    return $rows;
  }

  /**
   * Map.
   */
  public function map($set1, $set2) {
    $config = \Drupal::config('cmlapi.mapsettings');
    $map_sdandart = Yaml::parse($config->get($set1));
    $map_dop = Yaml::parse($config->get($set2));
    if (is_array($map_dop)) {
      $map = array_merge($map_sdandart, $map_dop);
    }
    else {
      $map = $map_sdandart;
    }
    return $map;
  }

}
