<?php

namespace Drupal\cmlapi\Service;

/**
 * Class ParserCatalog.
 */
class ParserCatalog extends ParserBase {

  /**
   * Parse.
   */
  public function parseFlatCatalog($cid = FALSE) {
    $data = FALSE;
    $uri = $this->cmlService->getFilePath($cid, 'import');
    if ($uri) {
      $this->xmlParserService->parseXmlFile($uri);
      $xml = $this->xmlParserService->xmlString;
      $data = $this->parceXml($xml, TRUE);
    }
    return $data;
  }

  /**
   * Parse.
   */
  public function parse($cid = FALSE) {
    $data = FALSE;
    $uri = $this->cmlService->getFilePath($cid, 'import');
    if ($uri) {
      $this->xmlParserService->parseXmlFile($uri);
      $xml = $this->xmlParserService->xmlString;
      $data = [
        'group' => $this->parceXml($xml, FALSE),
        'feature' => $this->parceFeature($xml),
        'category' => $this->parceCategory($xml),
      ];
    }
    return $data;
  }

  /**
   * Parce FilePath.
   */
  public function getRows($filepath, $skip_cache = FALSE) {
    $rows = &drupal_static("CatalogParcer::getRows():$filepath");
    if (!isset($rows)) {
      $cache_key = 'CatalogParcer:' . $filepath;
      if ($skip_cache) {
        $cache_key .= rand();
      }
      if ($cache = \Drupal::cache()->get($cache_key)) {
        $rows = $cache->data;
      }
      else {
        if ($filepath) {
          $this->xmlParserService->parseXmlFile($filepath);
          $data = $this->parceXml($this->xmlParserService->xmlString);
          if (!empty($data)) {
            $rows = $data;
          }
        }
        \Drupal::cache()->set($cache_key, $rows);
      }
    }
    return $rows;
  }

  /**
   * Parce.
   */
  public function parceCategory($xml) {
    $this->xmlParserService->parseXmlString($xml);
    $this->xmlParserService->get('import', 'category');
    return $this->xmlParserService->xmlfind;
  }

  /**
   * Parce.
   */
  public function parceFeature($xml) {
    $this->xmlParserService->parseXmlString($xml);
    $this->xmlParserService->get('import', 'feature');
    return $this->xmlParserService->xmlfind;
  }

  /**
   * Parce.
   */
  public function parceXml($xml, $flatTree = TRUE) {
    $this->xmlParserService->parseXmlString($xml);
    $this->xmlParserService->get('import', 'gruppa');
    if ($flatTree) {
      return $this->flatTree($this->xmlParserService->xmlfind);
    }
    else {
      return $this->xmlParserService->xmlfind;
    }
  }

  /**
   * Catalog flatTree.
   */
  public function flatTree(array $data, $parentId = NULL, $parent = TRUE) {
    $result = [];
    $i = 0;
    if (!empty($data)) {
      $data = $this->xmlParserService->arrayNormalize($data);
      foreach ($data as $key => $val) {
        $i++;
        $id = $val['Ид'];
        $result[$id] = [
          'id' => $val['Ид'],
          'name' => $val['Наименование'],
          'term_weight' => $i,
          'delete' => isset($val['ПометкаУдаления']) ? $val['ПометкаУдаления'] : FALSE,
        ];
        if ($parentId) {
          $result[$id]['parent'] = $parentId && !$parent ? $parentId : FALSE;
        }
        if (!empty($val['Группы']['Группа'])) {
          $result = array_merge($result, $this->flatTree($val['Группы']['Группа'], $id, FALSE));
        }
      }
    }
    return $result;
  }

}
